////////////////////////////////////////////////////////////
//Test SL013
//MCS51 source code
////////////////////////////////////////////////////////////      
#include <reg52.h>
#include <string.h>


#define OSC_FREQ         22118400L
#define TIME0_10ms       65536L - OSC_FREQ/1200L

#define  BOUND4800       256 - 24
#define  BOUND9600       256 - 12
#define  BOUND14400      256 - 8
#define  BOUND19200      256 - 6
#define  BOUND28800      256 - 4
#define  BOUND38400      256 - 3
#define  BOUND57600      256 - 2
#define  BOUND115200     256 - 1

//IO definition
sbit     LED       =    P2^0;                                                     

void InitializeSystem();                                                           
void SendCom(unsigned char *pCommand);                                           
void StartTime(unsigned int _MS);                                                  
void StopTime();

//Command List, length + command code + data frame
unsigned char code ComGetCsn[2]=        {0x02,0x10 };                                        

unsigned char code ComReadBlock1[10] =  {0x0A,0x11,0x00,0x01,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF};  

unsigned char code ComWriteBlock1[26] = {0x1A,0x12,0x00,0x01,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,
                                         0x00,0x11,0x22,0x33,0x44,0x55,0x66,0x77,0x88,0x99,
                                         0xAA,0xBB,0xCC,0xDD,0xEE,0xFF};                      
                               
unsigned char code ComIntiPurse2[14]  = {0x0E,0x13,0x00,0x02,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,                             
                                         0x78,0x56,0x34,0x12};                                
                               
unsigned char code ComReadPurse2[10]  = {0x0A,0x14,0x00,0x02,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF};  
                            
unsigned char code ComIncrPurse2[14]  = {0x0E,0x15,0x00,0x02,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,
                                         0x02,0x00,0x00,0x00};                                
                               
unsigned char code ComDecrPurse2[14]  = {0x0E,0x16,0x00,0x02,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,
                                         0x01,0x00,0x00,0x00};                                
              
//////////////////////////////////                               
unsigned char code pursevalue[4]={0x79,0x56,0x34,0x12};
                            
unsigned char g_cReceBuf[60];                                      
unsigned char g_cReceNum;                                          
unsigned char g_cCommand;                                          
unsigned char g_cStatus;                                           
unsigned char g_cWatchTime;                                        
unsigned char g_cCardType;                                         

bit   g_bReceCommandOk;                                            
bit   g_bReceAA;                                                   
bit   g_bSTI;
bit   g_bTimeOut;                                                  

void main() 
{
     InitializeSystem();
     
//Wait SL013 Reset  
     StartTime(100);                                       
     while(!g_bTimeOut);
     StopTime();

     while(1)
     {

         g_cCardType = 0xFF;
         SendCom(ComGetCsn);                               
         StartTime(8);                                     //waitting until receive response from SL013 or over 80ms 
         while (!g_bReceCommandOk && !g_bTimeOut);         //
         StopTime();                                       //
         if ((g_cCommand==0x10) && (g_cStatus==0) && !g_bTimeOut)
         {    g_cCardType = g_cReceBuf[7];   }             
          
         while ((g_cCardType == 0) || (g_cCardType == 1))  //Mifare_Std                                
         {

//Write block1 data: 0x00112233445566778899AABBCCDDEEFF 
             SendCom(ComWriteBlock1);                
             StartTime(8);
             while (!g_bReceCommandOk && !g_bTimeOut);
             StopTime();
             if ((g_cCommand!=0x12) || (g_cStatus!=0) || g_bTimeOut)
             {    break;    }               
             
//Read data from block1             
             SendCom(ComReadBlock1);                 
             StartTime(8);
             while (!g_bReceCommandOk && !g_bTimeOut);
             StopTime();
             if ((g_cCommand!=0x11) || (g_cStatus!=0) || g_bTimeOut)
             {    break;    }
//Compare data             
             if (memcmp(&ComWriteBlock1[10], &g_cReceBuf[3], 16) != 0)
             {    break;    }                 
             
             
//Initialize block2 to one pursevalue = 0x12345678 
             SendCom(ComIntiPurse2);                 
             StartTime(8);
             while (!g_bReceCommandOk && !g_bTimeOut);
             StopTime();
             if ((g_cCommand!=0x13) || (g_cStatus!=0) || g_bTimeOut)
             {    break;    }                 
             
             
//Incrment 2 to purse             
             SendCom(ComIncrPurse2);                 
             StartTime(8);
             while (!g_bReceCommandOk && !g_bTimeOut);
             StopTime();
             if ((g_cCommand!=0x15) || (g_cStatus!=0) || g_bTimeOut)
             {    break;    }      
             

//Decrment 1 to purse             
             SendCom(ComDecrPurse2);                 
             StartTime(8);
             while (!g_bReceCommandOk && !g_bTimeOut);
             StopTime();
             if ((g_cCommand!=0x16) || (g_cStatus!=0) || g_bTimeOut)
             {    break;    }  
              
                           
//Read purse value, should be equal to 0x12345679             
             SendCom(ComReadPurse2);                
             StartTime(8);
             while (!g_bReceCommandOk && !g_bTimeOut);
             StopTime();
             if ((g_cCommand!=0x14) || (g_cStatus!=0) || g_bTimeOut)
             {    break;    }
             if (memcmp(&pursevalue[0],&g_cReceBuf[3],4) != 0)
             {    break;    }                  


             LED = 0;                    
             StartTime(20);                                
             while(!g_bTimeOut);
             LED = 1;     
             StopTime();
         }
     }             
}

        
////////////////////////////////////////////////////////////
void InitializeSystem()
{
    TMOD &= 0x0F; 
    TMOD |= 0x21; 
    PCON |= 0x80;
    SCON  = 0x50; 
    TH1 = TL1 = BOUND19200; 
    TR1   = 1;
    IE   |= 0x90;
}

////////////////////////////////////////////////////////////
void StartTime(unsigned int _MS)
{
    TH0 = (unsigned char)((TIME0_10ms>>8)&0xFF);
    TL0 = (unsigned char)(TIME0_10ms&0xFF);
    g_cWatchTime = _MS;
    g_bTimeOut = 0;
    ET0 = 1;
    TR0 = 1;
}	

////////////////////////////////////////////////////////////
time0_int () interrupt 1 using 1
{
    TH0 = (unsigned char)((TIME0_10ms>>8) & 0xFF);
    TL0 = (unsigned char)(TIME0_10ms & 0xFF);
    if (g_cWatchTime--==0)
    {    g_bTimeOut = 1;    }
}

////////////////////////////////////////////////////////////
void StopTime()
{
    ET0 = 0;
    TR0 = 0;
}	

////////////////////////////////////////////////////////////
//Send command to SL013
////////////////////////////////////////////////////////////
void SendCom(unsigned char *pCommand)
{    
     unsigned char i,chkdata,sendleg;

     g_bReceCommandOk = 0;
     sendleg = *pCommand;
     
     g_bSTI = 0;
     SBUF = 0xAA;
     while (!g_bSTI);
     
     g_bSTI = 0;
     SBUF = 0xBB;
     while(!g_bSTI);
     
     chkdata = 0;
     for(i=0; i<sendleg; i++)
     {    
     	  chkdata ^= *(pCommand+i);
     	  
     	  g_bSTI = 0;
     	  SBUF = *(pCommand+i);
     	  while(!g_bSTI);
          
          if (*(pCommand+i) == 0xAA)                     
          {  
              g_bSTI = 0;
              SBUF = 0;
              while(!g_bSTI);
          }
     }
     
     g_bSTI = 0;
     SBUF = chkdata;
     while (!g_bSTI);
   
     if (chkdata == 0xAA)
     {  
         g_bSTI = 0;
         SBUF = 0;
         while (!g_bSTI);
     }
     
}

////////////////////////////////////////////////////////////
//Receive data from SL013
////////////////////////////////////////////////////////////
seri_int () interrupt 4 using 1
{
   unsigned char i;
   unsigned char rece_data = SBUF;
   unsigned char verify = 0;
   
   if(RI)
   {    
   	RI=0;
	if (g_bReceAA)                                      
	{  
	     g_bReceAA = 0;
	     switch (rece_data)
	     {    
	     	  case 0x00:
	               break;
	          case 0xbb:
	               g_cReceNum = 0;
	               break;
	          default:
	               i = g_cReceNum;
	               g_cReceBuf[i] = rece_data;
	               break;
	     }
	}
	else
	{    
	     switch(rece_data)
	     {    
	     	  case 0xaa:
	               g_bReceAA = 1;
	          default:
	               i = g_cReceNum++;
	               g_cReceBuf[i] = rece_data;
	               break;
	     }
	}
	
	i = g_cReceBuf[0];                                 
        if(g_cReceNum==i+1)                                 
        {   
            for (i=0; i<g_cReceBuf[0]+1; i++)
            {
            	verify ^= g_cReceBuf[i];
            }
            if(!verify)                                     
            {   
                g_bReceCommandOk = 1;
                g_cCommand = g_cReceBuf[1];
                g_cStatus = g_cReceBuf[2]; 
                g_cReceNum = 0;
            }
        }
        
        if (g_cReceNum > sizeof(g_cReceBuf))                
        {    g_cReceNum = 0;     }
    }
    
    if(TI)
    {   
        TI = 0; 
        g_bSTI = 1; 
    }
}
